/**
 * JQuery Mobile Swipe
 *
 * This plugin adds some classes to html's DOM which can be used as fallback for action such as :hover which are not
 * always intuitive on mobile ( and sometimes, unexisting ).

 * Based on jquery.detectSwipe v2.1.1 [http://github.com/marcandre/detect_swipe] which is based on touchwipe by Andreas Waltl, netCU Internetagentur (http://www.netcu.de)
 */
(function ($) {
    $.mobileswipe = {
        attach      : register,
        enabled     : 'ontouchstart' in document.documentElement,
        threshold   : 20,
        reset_timer : null
    };

    $.fn.mobileswipe = register;

    $.event.special.swipe = { setup: register };

    $.each(['left', 'up', 'down', 'right'], function () {
        $.event.special['swipe' + this] = {setup: function () {
                $(this).on('swipe', $.noop);
            }};
    });

    $(function() { $('.swipe').mobileswipe(); });

    var startX, startY,
        timeout_instance = [],
        isMoving = false;

    function onTouchEnd() {
        this.removeEventListener('touchmove', onTouchMove);
        this.removeEventListener('touchend', onTouchEnd);
        isMoving = false;
    }

    function onTouchMove(e) {
        if (isMoving) {
            var dir, delta, $this = $(this);

            if (Math.abs(delta = startX - e.touches[0].pageX) >= $.mobileswipe.threshold) {
                dir = delta > 0 ? 'left' : 'right';
                $this.removeClass('swipe-left swipe-right');
            }
            else if (Math.abs(delta = startY - e.touches[0].pageY) >= $.mobileswipe.threshold) {
                dir = delta < 0 ? 'down' : 'up';
                $this.removeClass('swipe-down swipe-up');
            }

            if ( dir )  {
                onTouchEnd.call(this);
                $this.trigger('swipe', dir).trigger('swipe' + dir).addClass('swipe-' + dir);
                checkPersistency($this, dir, $this.data('reset-timer'));
            }
        }
    }

    function onTouchStart(e) {
        if (e.touches.length === 1) {
            startX = e.touches[0].pageX;
            startY = e.touches[0].pageY;
            isMoving = true;

            this.addEventListener('touchmove', onTouchMove, false);
            this.addEventListener('touchend', onTouchEnd, false);
        }
    }

    function checkPersistency($elem, dir, timer) {
        timer || ( timer = $.mobileswipe.reset_timer );

        if ( timer ) {
            (dir in timeout_instance) && clearTimeout(timeout_instance[dir]);

            timeout_instance[dir] = setTimeout(function() {
                $elem.removeClass('swipe-'+dir);
            }, timer);
        }
    }

    function register() {
        $(this).each(function() {
            this.addEventListener && this.addEventListener('touchstart', onTouchStart, false);
        });
    }

    function unregister() {
        $(this).each(function() {
            this.removeEventListener('touchstart', onTouchStart);
        });
    }
})(jQuery);